#include <EEPROMAnything.h>
#include <EEPROM.h>
#include <IRremote.h>
#include <avr/pgmspace.h>
#include <SoftwareSerial.h>
/*
 * This program uses Ken Sherriff's library and IRrecord
 * example modified.
 */

SoftwareSerial BTSerial(6, 7);

const int RECV_PIN PROGMEM = 11;
const int BUTTON_PIN PROGMEM = 12;
const int STATUS_PIN PROGMEM = 13;

IRrecv irrecv(RECV_PIN);
IRsend irsend;

String inputString = "";     
boolean stringComplete = false;

decode_results results;

struct IRcode
{
  int codeType = -1;
  unsigned long codeValue;
  unsigned int rawCodes[RAWBUF]; 
  int codeLen; 
  int toggle = 0; 
};
typedef struct IRcode IRCode;

IRCode ircodes[4];

void setup()
{
  BTSerial.begin(9600);
  inputString.reserve(100);
  Serial.begin(9600);
  EEPROM_readAnything(0, ircodes);
  Serial.println(F("Starting..."));
  pinMode(BUTTON_PIN, INPUT);
  pinMode(STATUS_PIN, OUTPUT);
  Serial.println(F("Started"));
}

void storeCode(decode_results *results, int i) {
  ircodes[i].codeType = results->decode_type;
  int count = results->rawlen;
  if (ircodes[i].codeType == UNKNOWN) {
    Serial.println(F("Received unknown code, saving as raw"));
    ircodes[i].codeLen = results->rawlen - 1;
    for (int y = 1; y <= ircodes[i].codeLen; y++) {
      if (y % 2) {
        ircodes[i].rawCodes[y - 1] = results->rawbuf[y]*USECPERTICK - MARK_EXCESS;
        Serial.print(F(" m"));
      } 
      else {
        ircodes[i].rawCodes[y - 1] = results->rawbuf[y]*USECPERTICK + MARK_EXCESS;
        Serial.print(F(" s"));
      }
      Serial.print(ircodes[i].rawCodes[y - 1], DEC);
    }
    Serial.println("");
  }
  else {
    if (ircodes[i].codeType == NEC) {
      Serial.print(F("Received NEC: "));
      if (results->value == REPEAT) {
        Serial.println(F("repeat; ignoring."));
        return;
      }
    } 
    else if (ircodes[i].codeType == SONY) {
      Serial.print(F("Received SONY: "));
    } 
    else if (ircodes[i].codeType == RC5) {
      Serial.print(F("Received RC5: "));
    } 
    else if (ircodes[i].codeType == RC6) {
      Serial.print(F("Received RC6: "));
    } 
    else {
      Serial.print(F("Unexpected ircodes[i].codeType "));
      Serial.print(ircodes[i].codeType, DEC);
      Serial.println("");
    }
    Serial.println(results->value, HEX);
    ircodes[i].codeValue = results->value;
    ircodes[i].codeLen = results->bits;
  }
}

void sendCode(int repeat, int i) {
  if (ircodes[i].codeType == NEC) {
    if (repeat) {
      irsend.sendNEC(REPEAT, ircodes[i].codeLen);
      Serial.println(F("Sent NEC repeat"));
    } 
    else {
      irsend.sendNEC(ircodes[i].codeValue, ircodes[i].codeLen);
      Serial.print(F("Sent NEC "));
      Serial.println(ircodes[i].codeValue, HEX);
    }
  } 
  else if (ircodes[i].codeType == SONY) {
    irsend.sendSony(ircodes[i].codeValue, ircodes[i].codeLen);
    Serial.print(F("Sent Sony "));
    Serial.println(ircodes[i].codeValue, HEX);
  } 
  else if (ircodes[i].codeType == RC5 || ircodes[i].codeType == RC6) {
    if (!repeat) {
      ircodes[i].toggle = 1 - ircodes[i].toggle;
    }
    ircodes[i].codeValue = ircodes[i].codeValue & ~(1 << (ircodes[i].codeLen - 1));
    ircodes[i].codeValue = ircodes[i].codeValue | (ircodes[i].toggle << (ircodes[i].codeLen - 1));
    if (ircodes[i].codeType == RC5) {
      Serial.print(F("Sent RC5 "));
      Serial.println(ircodes[i].codeValue, HEX);
      irsend.sendRC5(ircodes[i].codeValue, ircodes[i].codeLen);
    } 
    else {
      irsend.sendRC6(ircodes[i].codeValue, ircodes[i].codeLen);
      Serial.print(F("Sent RC6 "));
      Serial.println(ircodes[i].codeValue, HEX);
    }
  } 
  else if (ircodes[i].codeType == UNKNOWN /* i.e. raw */) {
    irsend.sendRaw(ircodes[i].rawCodes, ircodes[i].codeLen, 38);
    Serial.println(F("Sent raw"));
  }
}

int lastButtonState;
void(* resetFunc) (void) = 0;
void loop() {
  int i = 0;
  if(stringComplete){
    if(inputString.startsWith("record")){
      i = inputString.substring(7).toInt();
      irrecv.enableIRIn();
      Serial.print(F("Listening... "));
      Serial.println(i);
      while(true){
         if (irrecv.decode(&results)) {
            digitalWrite(STATUS_PIN, HIGH);
            storeCode(&results, i);
            irrecv.resume();
            digitalWrite(STATUS_PIN, LOW);
            EEPROM_writeAnything(0, ircodes);
            break;
         }
      }
    }
    else if(inputString == "EEPROM_CLEAR\n"){
      Serial.println(F("Clearing EEPROM..."));
      eeprom_clear();
      Serial.println(F("Restarting..."));
      Serial.flush();
      resetFunc();
    }  
    else if(inputString.startsWith("AT")){
      BTSerial.print(inputString.substring(0, inputString.indexOf('\n')));
      Serial.println("OK");
    }
    stringComplete = false;
    inputString = "";
    
  }
  if(BTSerial.available()){
    String input2 = "";
     input2 = BTSerial.readStringUntil('n');
    if(input2.startsWith("play")) {
      i = input2.substring(5).toInt();
      Serial.print(F("Recieved, sending "));
      Serial.println(i);
      digitalWrite(STATUS_PIN, HIGH);
      sendCode(false, i);
      digitalWrite(STATUS_PIN, LOW);
      delay(50);
    }
    input2 = "";
  }
 
}

void serialEvent() {
  while (Serial.available()) {
    char inChar = (char)Serial.read();
    inputString += inChar;
    if (inChar == '\n') {
      stringComplete = true;
    }
  }
}
void eeprom_clear(){
  for (int i = 0 ; i < EEPROM.length() ; i++) {
    EEPROM.write(i, 0);
  }
}

